package org.tzambalayev.ideaplugins.tabswitch;

import java.awt.Color;
import javax.swing.Icon;

import com.intellij.openapi.editor.ScrollType;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileEditor.OpenFileDescriptor;
import com.intellij.openapi.fileTypes.FileTypeManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.FileStatusManager;
import com.intellij.openapi.vfs.VirtualFile;

import org.intellij.openapi.fileTypes.FileTypeManagerWrapper;
import org.intellij.openapi.fileTypes.FileTypeWrapper;

final class TabSelector implements TabSelectorInterface {

    private final VirtualFile[] files;
    private final VirtualFile[] openFiles;
    private final FileEditorManager fileEditorManager;
    private final FileStatusManager fileStatusManager;

    TabSelector(Project project) {
        TabSwitchProjectComponent tabSwitchProjectComponent =
                (TabSwitchProjectComponent) project.getComponent(TabSwitchProjectComponent.class);
        files = tabSwitchProjectComponent.getFiles();
        openFiles = tabSwitchProjectComponent.getOpenFiles();
        fileEditorManager = FileEditorManager.getInstance(project);
        fileStatusManager = FileStatusManager.getInstance(project);
    }

    String[] getFileNames() {
        String[] fileNames = new String[files.length];
        for (int i = 0; i < fileNames.length; i++) {
            fileNames[i] = files[i].getName();
        }
        return fileNames;
    }

    public void selected(int selectedIndex) {
        if (selectedIndex > 0 || openFiles.length == 0) {
            VirtualFile virtualFile = files[selectedIndex];
            if (virtualFile.isValid()) {
                fileEditorManager.openFile(new OpenFileDescriptor(virtualFile), ScrollType.MAKE_VISIBLE, true);
            }
        }
    }

    FileEntry[] getFileEntries() {
        FileEntry[] fileEntries = new FileEntry[files.length];
        FileTypeManager fileTypeManager = FileTypeManagerWrapper.getInstance();
        for (int i = 0; i < fileEntries.length; i++) {
            VirtualFile virtualFile = files[i];
            FileTypeWrapper fileTypeWrapper = (FileTypeWrapper) fileTypeManager.getFileTypeByFile(virtualFile);
            Icon icon = fileTypeWrapper.getIcon(virtualFile);
            Color statusColor = fileStatusManager.getStatusColor(virtualFile);
            fileEntries[i] = new FileEntry(icon, statusColor);
        }
        return fileEntries;
    }

}
