package org.tzambalayev.ideaplugins.tabswitch;

import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileEditor.FileEditorManagerEvent;
import com.intellij.openapi.fileEditor.FileEditorManagerListener;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;

import org.intellij.openapi.components.ProjectComponentAdapter;

final class TabSwitchProjectComponent extends ProjectComponentAdapter {

    private final Project project;
    private final VirtualFileMRUStack virtualFileMRUStack;

    private FileEditorManager fileEditorManager;

    TabSwitchProjectComponent(Project project) {
        this.project = project;
        virtualFileMRUStack = new VirtualFileMRUStack(TabSwitchApplicationComponent.getInstance().getHistoryLimit());
    }

    public String getComponentName() {
        return "TabSwitchProjectComponent";
    }

    public void initComponent() {
        fileEditorManager = FileEditorManager.getInstance(project);
        FileEditorManagerListener fileEditorManagerListener = new FileEditorManagerListener() {
            public void selectedFileChanged(FileEditorManagerEvent event) {
                virtualFileMRUStack.push(event.getNewFile());
            }
        };
        fileEditorManager.addFileEditorManagerListener(fileEditorManagerListener);
    }

    VirtualFile[] getFiles() {
        virtualFileMRUStack.checkCurrentList(getOpenFiles());
        return virtualFileMRUStack.getFiles();
    }

    VirtualFile[] getOpenFiles() {
        return fileEditorManager.getOpenFiles();
    }

}
