package org.tzambalayev.ideaplugins.tabswitch;

import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

public final class MRUStackTest extends TestCase {

    private MRUStack mruStack;
    private List list;

    protected void setUp() {
        create(0);
    }

    private void create(int recentMin) {
        mruStack = new MRUStack(recentMin);
        list = mruStack.getList();
    }

    private void checkList(String[] strings) {
        assertEquals(Arrays.asList(strings), list);
    }

    public void testPush() {
        mruStack.push("a");
        checkList(new String[] {"a"});
    }

    public void testAddAtTheBeginning() {
        mruStack.push("a");
        mruStack.push("b");
        checkList(new String[] {"b", "a"});
    }

    public void testNoDuplicates() {
        mruStack.push("a");
        mruStack.push("b");
        mruStack.push("a");
        checkList(new String[] {"a", "b"});
    }

    public void testRetainAll() {
        mruStack.push("a");
        mruStack.push("b");
        String[] strings = new String[] {"a"};
        mruStack.checkCurrentList(strings);
        checkList(strings);
    }

    public void testPushNull() {
        try {
            mruStack.push(null);
            fail();
        } catch (NullPointerException e) {
        }
    }

    public void testRecentMin() {
        create(1);
        mruStack.push("a");
        mruStack.checkCurrentList(new String[0]);
        checkList(new String[] {"a"});
    }

}
