package org.tzambalayev.ideaplugins.tabswitch;

import com.intellij.openapi.vfs.VirtualFile;
import junit.framework.TestCase;

import org.intellij.openapi.vfs.VirtualFileUnsupportedAdapter;

public final class VirtualFileMRUStackTest extends TestCase {

    private VirtualFileMRUStack virtualFileMRUStack;

    private static void assertEquals(VirtualFile[] expected, VirtualFile[] actual) {
        assertEquals(expected.length, actual.length);
        for (int i = 0; i < expected.length; i++) {
            assertEquals(expected[i], actual[i]);
        }
    }

    protected void setUp() {
        virtualFileMRUStack = new VirtualFileMRUStack(1);
    }

    public void testGetList() {
        VirtualFile newFile = new VirtualFileUnsupportedAdapter() {
            public boolean isValid() {
                return true;
            }
        };
        VirtualFile[] openFiles = new VirtualFile[] {newFile};
        virtualFileMRUStack.push(newFile);
        virtualFileMRUStack.checkCurrentList(openFiles);
        VirtualFile[] virtualFiles = virtualFileMRUStack.getFiles();
        assertEquals(openFiles, virtualFiles);
    }

    public void testPushNull() {
        VirtualFile[] openFiles = new VirtualFile[0];
        virtualFileMRUStack.push(null);
        virtualFileMRUStack.checkCurrentList(openFiles);
        VirtualFile[] virtualFiles = virtualFileMRUStack.getFiles();
        assertEquals(openFiles, virtualFiles);
    }

    public void testCheckCurrentList() {
        VirtualFile newFile = new VirtualFileUnsupportedAdapter() {
            public boolean isValid() {
                return false;
            }
        };
        VirtualFile[] openFiles = new VirtualFile[] {newFile};
        virtualFileMRUStack.push(newFile);
        virtualFileMRUStack.checkCurrentList(openFiles);
        VirtualFile[] virtualFiles = virtualFileMRUStack.getFiles();
        assertEquals(0, virtualFiles.length);
    }

}
