package org.intellij.codeInsight.intention;

import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiElementFactory;
import com.intellij.psi.PsiFile;
import com.intellij.psi.PsiManager;
import com.intellij.util.IncorrectOperationException;

import org.intellij.psi.PsiElementWrapper;

public abstract class Intention implements IntentionAction {

    private Project project;
    private Editor editor;
    private PsiFile psiFile;
    private PsiElement acceptedLeaf;

    protected Intention() {
    }

    protected final PsiElement getAcceptedLeaf() {
        return acceptedLeaf;
    }

    protected final PsiElementFactory getElementFactory() {
        return PsiManager.getInstance(project).getElementFactory();
    }

    protected final Editor getEditor() {
        return editor;
    }

    public final boolean isAvailable(Project project, Editor editor, PsiFile psiFile) {
        this.project = project;
        this.editor = editor;
        this.psiFile = psiFile;
        return isAvailable();
    }

    public final void invoke(Project project, Editor editor, PsiFile psiFile) throws IncorrectOperationException {
        if (this.project != project || this.editor != editor || this.psiFile != psiFile || !isAvailable()) {
            throw new RuntimeException();
        }
        invoke();
    }

    protected abstract PsiElement isAvailable(PsiElement psiElement);

    protected abstract void invoke() throws IncorrectOperationException;

    private boolean isAvailable() {
        acceptedLeaf = findAcceptedLeaf();
        return acceptedLeaf != null;
    }

    private PsiElement findAcceptedLeaf() {
        PsiElement acceptedLeaf = null;
        int offset = editor.getCaretModel().getOffset();
        PsiElement elementAtOffset = psiFile.findElementAt(offset);
        if (elementAtOffset != null) {
            PsiElementWrapper psiElementWrapper = new PsiElementWrapper(elementAtOffset);
            PsiElement[] psiElements = psiElementWrapper.findLeavesAt(offset);
            for (int i = 0; i < psiElements.length; i++) {
                acceptedLeaf = isAvailable(psiElements[i]);
                if (acceptedLeaf != null) {
                    break;
                }
            }
        }
        return acceptedLeaf;
    }

}
