package org.intellij.codeInsight.intention;

import com.intellij.openapi.project.Project;

import org.intellij.openapi.application.ApplicationInfoWrapper;

public abstract class IntentionManager {

    private static final IntentionManagerMap INTENTION_MANAGER_MAP;

    static {
        IntentionManagerFactory intentionManagerFactory;
        if (ApplicationInfoWrapper.getWrapperInstance().buildNumber() < 818) {
            intentionManagerFactory = new AriadnaIntentionManagerFactory();
        } else {
            intentionManagerFactory = new AuroraIntentionManagerFactory();
        }
        INTENTION_MANAGER_MAP = new IntentionManagerMap(intentionManagerFactory);
    }

    public static IntentionManager getInstance(Project project) {
        return INTENTION_MANAGER_MAP.getIntentionManager(project);
    }

    public abstract void addAction(IntentionAction intentionAction);

    public abstract IntentionAction[] getIntentionActions();

}
