package org.intellij.openapi.actionSystem;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;

public final class ActionManagerUtil {

    private ActionManagerUtil() {
    }

    private static ActionManager getActionManager() {
        return ActionManager.getInstance();
    }

    private static Map unregister(String[] actionIds) {
        Map actionMap = new HashMap(actionIds.length);
        ActionManager actionManager = getActionManager();
        for (int i = 0; i < actionIds.length; i++) {
            String actionId = actionIds[i];
            AnAction action = actionManager.getAction(actionId);
            if (action == null) {
                throw new NullPointerException(actionId);
            }
            actionMap.put(actionId, action);
            actionManager.unregisterAction(actionId);
        }
        return actionMap;
    }

    private static void register(Map actionMap) {
        ActionManager actionManager = getActionManager();
        for (Iterator iterator = actionMap.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry entry = (Map.Entry) iterator.next();
            actionManager.registerAction((String) entry.getKey(), (AnAction) entry.getValue());
        }
    }

    public static void registerActions(String[] actionIdsToRegister, String[] actionIdsToUnregister) {
        Map actionsToRegister = unregister(actionIdsToRegister);
        Map actionsToUnregister = unregister(actionIdsToUnregister);
        register(actionsToRegister);
        register(actionsToUnregister);
    }

}
