package org.intellij.openapi.editor;

import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.LogicalPosition;
import com.intellij.openapi.util.TextRange;

import org.intellij.openapi.util.OffsetRange;

public final class SelectionModelWrapper extends SelectionModelDelegate {

    private final Editor editor;
    private final DocumentWrapper documentWrapper;
    private final CaretModel caretModel;

    SelectionModelWrapper(Editor editor, DocumentWrapper documentWrapper) {
        super(editor.getSelectionModel());
        this.editor = editor;
        this.documentWrapper = documentWrapper;
        caretModel = editor.getCaretModel();
    }

    private int getLineNumber(int offset) {
        return documentWrapper.getLineNumber(offset);
    }

    public int getSelectionStartLineNumber() {
        return getLineNumber(getSelectionStart());
    }

    public int getSelectionEndLineNumber() {
        return getLineNumber(getSelectionEnd() - 1) + 1;
    }

    public void shiftStartSelection(int offset) {
        setSelection(getSelectionStart() + offset, getSelectionEnd());
    }

    public LineRange getSelectedLines() {
        LineRange lineRange = null;
        if (hasSelection()) {
            int selectionStart = getSelectionStart();
            int selectionEnd = getSelectionEnd();
            if (documentWrapper.isLineStart(selectionStart) && documentWrapper.isLineStart(selectionEnd)) {
                int startLineNumber = documentWrapper.getLineNumber(selectionStart);
                int endLineNumber = documentWrapper.getLineNumber(selectionEnd);
                lineRange = new LineRange(startLineNumber, endLineNumber);
            }
        }
        return lineRange;
    }

    public void selectLines(LineRange lineRange) {
        int startOffset = documentWrapper.getLineStartOffset(lineRange.getStartLineNumber());
        int endOffset = documentWrapper.getLineStartOffset(lineRange.getEndLineNumber());
        setSelection(startOffset, endOffset);
    }

    public void select(LogicalPosition startPosition, LogicalPosition endPosition) {
        int startOffset = editor.logicalPositionToOffset(startPosition);
        int endOffset = editor.logicalPositionToOffset(endPosition);
        setSelection(startOffset, endOffset);
    }

    public void moveSelectionToOffset(int offset) {
        setSelection(getLeadSelectionOffset(), offset);
        caretModel.moveToOffset(offset);
    }

    public OffsetRange getSelectionRange() {
        return new OffsetRange(getSelectionStart(), getSelectionEnd());
    }

    public void setSelection(TextRange textRange) {
        setSelection(textRange.getStartOffset(), textRange.getEndOffset());
    }

    public void selectAll() {
        setSelection(documentWrapper.getAllSelection());
    }

}
