package org.intellij.openapi.util;

import com.intellij.openapi.util.TextRange;

public final class OffsetRange extends TextRange {

    public static final OffsetRange INVALID = new OffsetRange(-1, -1);

    public OffsetRange(int startOffset, int endOffset) {
        super(startOffset, endOffset);
    }

    public boolean isValid() {
        int startOffset = getStartOffset();
        int endOffset = getEndOffset();
        return startOffset >= 0 && endOffset >= 0 && startOffset < endOffset;
    }

    public boolean isEmpty() {
        return getStartOffset() >= getEndOffset();
    }

    public boolean contains(int offset) {
        return getStartOffset() <= offset && offset <= getEndOffset();
    }

    public boolean isLess(TextRange textRange) {
        return !isGreaterOrEquals(textRange);
    }

    public boolean isGreaterOrEquals(TextRange textRange) {
        return contains(textRange);
    }

    public boolean isGreater(TextRange textRange) {
        return isGreaterOrEquals(textRange) && !equals(textRange);
    }

    public boolean isLessOrEquals(TextRange textRange) {
        return !isGreater(textRange);
    }

}
