package org.intellij.openapi.util;

public final class ReadOnlyCharArray {

    private final char[] chars;
    private final int length;

    private String string;

    public ReadOnlyCharArray(char[] chars, int length) {
        this.chars = chars;
        this.length = length;
    }

    public String substring(int start, int end) {
        return new String(chars, start, end - start);
    }

    public String newString(int offset, int count) {
        return substring(offset, offset + count);
    }

    private String getString() {
        if (string == null) {
            string = new String(chars, 0, length);
        }
        return string;
    }

    public int indexOf(String s, int fromIndex) {
        return getString().indexOf(s, fromIndex);
    }

    public int lastIndexOf(String s, int fromIndex) {
        return getString().lastIndexOf(s, fromIndex);
    }

    public char charAt(int index) {
        if (index >= length) {
            throw new IndexOutOfBoundsException();
        }
        return chars[index];
    }

    public int length() {
        return length;
    }

}
