package org.intellij.openapi.util.externalizer;

import java.util.List;

import com.intellij.openapi.diagnostic.Logger;
import org.jdom.Element;

public final class JDOMExternalizer {

    private static final Logger LOGGER = Logger.getInstance("org.intellij.openapi.util.externalizer.JDOMExternalizer");

    private static final String OPTION = "option";
    private static final String NAME = "name";
    private static final String VALUE = "value";

    private JDOMExternalizer() {
    }

    private static FieldItem[] getFieldItems(Element element) {
        List children = element.getChildren(OPTION);
        FieldItem[] fieldItems = new FieldItem[children.size()];
        for (int i = 0; i < fieldItems.length; i++) {
            Element child = (Element) children.get(i);
            String name = child.getAttributeValue(NAME);
            String value = child.getAttributeValue(VALUE);
            log("read", name, value);
            fieldItems[i] = new FieldItem(name, value);
        }
        return fieldItems;
    }

    public static void writeExternal(Object object, Element element) {
        try {
            FieldItem[] fieldItems = JDOMWriter.getFieldItems(object);
            for (int i = 0; i < fieldItems.length; i++) {
                FieldItem fieldItem = fieldItems[i];
                Element child = new Element(OPTION);
                element.addContent(child);
                String name = fieldItem.getName();
                String value = fieldItem.getText();
                log("write", name, value);
                child.setAttribute(NAME, name);
                child.setAttribute(VALUE, value);
            }
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    public static void readExternal(Object object, Element element) {
        try {
            FieldItem[] fieldItems = getFieldItems(element);
            for (int i = 0; i < fieldItems.length; i++) {
                JDOMReader.populateField(object, fieldItems[i]);
            }
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    private static void log(String operation, String name, String value) {
        LOGGER.info(operation + ", name: '" + name + "', value: '" + value + "'");
    }

}
