package org.intellij.openapi.util.externalizer;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

final class JDOMWriter {

    private JDOMWriter() {
    }

    private static boolean isSerializableField(Field field) {
        int modifiers = field.getModifiers();
        return !Modifier.isStatic(modifiers) && !Modifier.isTransient(modifiers) && !Modifier.isFinal(modifiers);
    }

    static FieldItem[] getFieldItems(Object object) {
        List fieldItemList = new ArrayList();
        Field[] fields = object.getClass().getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            if (isSerializableField(field)) {
                fieldItemList.add(getFieldItem(object, field));
            }
        }
        return (FieldItem[]) fieldItemList.toArray(new FieldItem[fieldItemList.size()]);
    }

    private static FieldItem getFieldItem(Object object, Field field) {
        Value value = ValueFactory.getValue(field.getType());
        field.setAccessible(true);
        try {
            value.get(object, field);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
        return new FieldItem(field.getName(), value);
    }

}
