package org.intellij.psi;

import java.util.ArrayList;
import java.util.List;

import com.intellij.openapi.util.TextRange;
import com.intellij.psi.PsiElement;

public final class PsiElementWrapper extends PsiElementDelegate {

    public PsiElementWrapper(PsiElement psiElement) {
        super(psiElement);
    }

    private static PsiElement[] toPsiElementArray(List psiElementList) {
        return (PsiElement[]) psiElementList.toArray(new PsiElement[psiElementList.size()]);
    }

    public PsiElement[] findLeavesAt(int offset) {
        List list = new ArrayList();
        PsiElement psiElement = findElementAt(offset);
        if (psiElement != null) {
            list.add(psiElement);
            if (psiElement.getTextOffset() == offset) {
                PsiElement prevLeaf = new PsiElementWrapper(psiElement).getPrevLeaf();
                if (prevLeaf != null) {
                    TextRange textRange = prevLeaf.getTextRange();
                    if (textRange != null && textRange.getEndOffset() == offset) {
                        list.add(prevLeaf);
                    }
                }
            }
        }
        return toPsiElementArray(list);
    }

    private PsiElement getPrevLeaf() {
        PsiElement prevLeaf;
        PsiElement prevSibling = getPrevSibling();
        if (prevSibling == null) {
            prevLeaf = new PsiElementWrapper(getParent()).getPrevLeaf();
        } else {
            PsiElement[] children = prevSibling.getChildren();
            int length = children.length;
            if (length == 0) {
                prevLeaf = prevSibling;
            } else {
                prevLeaf = children[length - 1];
            }
        }
        return prevLeaf;
    }

}
