package org.intellij.psi.codeStyle;

import java.util.Iterator;

import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.codeStyle.CodeStyleManager;
import com.intellij.util.IncorrectOperationException;

import org.intellij.openapi.editor.EditorWrapper;
import org.intellij.openapi.util.OffsetRange;

public final class CodeStyleManagerWrapper extends CodeStyleManagerDelegate {

    public CodeStyleManagerWrapper(CodeStyleManager codeStyleManager) {
        super(codeStyleManager);
    }

    private static void notifyReformatListeners(ReformatEvent reformatEvent) {
        for (Iterator iterator = ReformatManager.getInstance().getReformatListeners().iterator(); iterator.hasNext();) {
            ((ReformatListener) iterator.next()).psiElementReformatted(reformatEvent);
        }
    }

    public void reformatRange(PsiElement psiElement, OffsetRange offsetRange, EditorWrapper editor, DataContext dataContext) {
        PsiElement result;
        try {
            result = reformatRange(psiElement, offsetRange.getStartOffset(), offsetRange.getEndOffset());
        } catch (IncorrectOperationException e) {
            throw new RuntimeException(e);
        }
        notifyReformatListeners(new ReformatRangeEvent(result, editor.getNativeEditor(), dataContext, offsetRange));
    }

    public void wrappedOptimizeImports(PsiFile psiFile) {
        try {
            optimizeImports(psiFile);
        } catch (IncorrectOperationException e) {
            throw new RuntimeException(e);
        }
    }

}
