package org.intellij.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public final class ReflectUtil {

    public static final Class[] EMPTY_CLASS_ARRAY = new Class[0];

    private static final Object[] EMPTY_OBJECT_ARRAY = new Object[0];

    private ReflectUtil() {
    }

    public static Object invoke(Method method, Object object) {
        return invoke(method, object, EMPTY_OBJECT_ARRAY);
    }

    public static Object invoke(Method method, Object object, Object[] parameters) {
        try {
            return method.invoke(object, parameters);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            Throwable cause;
            if (e == null) {
                cause = e;
            } else {
                cause = e.getCause();
            }
            throw new RuntimeException(cause);
        }
    }

    public static Method getMethod(Class aClass, String name) {
        return getMethod(aClass, name, EMPTY_CLASS_ARRAY);
    }

    public static Method getMethod(Class aClass, String name, Class[] parameterTypes) {
        try {
            return aClass.getMethod(name, parameterTypes);
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }

    public static Method getDeclaredMethod(Class aClass, String name, Class[] parameterTypes) {
        try {
            Method declaredMethod = aClass.getDeclaredMethod(name, parameterTypes);
            declaredMethod.setAccessible(true);
            return declaredMethod;
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }

    public static Field getDeclaredField(Class aClass, String name) {
        try {
            Field declaredField = aClass.getDeclaredField(name);
            declaredField.setAccessible(true);
            return declaredField;
        } catch (NoSuchFieldException e) {
            throw new RuntimeException(e);
        }
    }

    public static Object get(Field field, Object object) {
        try {
            return field.get(object);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    public static Class getClassForName(String className) {
        try {
            return Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

}
