package org.intellij.psi.codeStyle;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import junit.framework.TestCase;

public final class ReformatManagerTest extends TestCase {

    private ReformatManager reformatManager;

    protected void setUp() {
        reformatManager = ReformatManager.getInstance();
    }

    public void testGetInstance() {
        assertNotNull(reformatManager);
    }

    public void testIsActive() {
        assertTrue(!reformatManager.isActive());
        reformatManager.activate();
        assertTrue(reformatManager.isActive());
    }

    public void testGetReformatListeners() {
        Collection reformatListeners = reformatManager.getReformatListeners();
        assertNotNull(reformatListeners);
        assertSame(reformatListeners, reformatManager.getReformatListeners());
        try {
            reformatListeners.add("");
            fail();
        } catch (UnsupportedOperationException e) {
        }
    }

    public void testAddReformatListener() {
        Set set = new HashSet();
        for (int i = 0; i < 2; i++) {
            ReformatListener reformatListener = new ReformatListener() {
                public void psiElementReformatted(ReformatEvent reformatEvent) {
                }
            };
            reformatManager.addReformatListener(reformatListener);
            set.add(reformatListener);
        }
        assertEquals(set, new HashSet(reformatManager.getReformatListeners()));
        try {
            reformatManager.addReformatListener(null);
            fail();
        } catch (NullPointerException e) {
        }
    }

}
